eisl -s verify/array.lsp
eisl -s verify/bignum.lsp
eisl -s verify/char.lsp
eisl -s verify/cond.lsp
eisl -s verify/block.lsp
eisl -s verify/control.lsp
eisl -s verify/declare.lsp
eisl -s verify/elixir.lsp
eisl -s verify/files.lsp
eisl -s verify/formeval.lsp
eisl -s verify/hiroi.lsp
eisl -s verify/ilos.lsp
eisl -s verify/io.lsp
eisl -s verify/list.lsp
eisl -s verify/macro.lsp
eisl -s verify/matrix.lsp
eisl -s verify/misc.lsp
eisl -s verify/noble.lsp
eisl -s verify/number.lsp
eisl -s verify/object.lsp
eisl -s verify/pred.lsp
eisl -s verify/seq.lsp
eisl -s verify/stream.lsp
eisl -s verify/string.lsp
eisl -s verify/symbol.lsp
eisl -s verify/unicode.lsp
eisl -s verify/unwind.lsp
eisl -s verify/vector.lsp
eisl -s verify/arvyy.lsp