void init_declare(void){int th=0;
signal(SIGINT, signal_handler_c);
Fsetcatchsymbols(NIL);Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-PARENTS"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(0),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-SONS"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(1),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-SN"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(2),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-ENTRY1"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(3),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-ENTRY2"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(4),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-ENTRY3"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(5),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-ENTRY4"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(6),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-ENTRY5"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(7),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-ENTRY6"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(8),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(T,NIL)),0);
Fpeval(Fcons(Fmakesym("DEFMACRO"),Fcons(Fmakesym("NODE-MARK"),Fcons(Fcons(Fmakesym("X"),NIL),Fcons(Fcons(Fmakesym("QUASI-QUOTE"),Fcons(Fcons(Fmakesym("ELT"),Fcons(Fcons(Fmakesym("UNQUOTE"),Fcons(Fmakesym("X"),NIL)),Fcons(Fmakeint(9),NIL))),NIL)),NIL)))),0);
Fpeval(Fcons(Fmakesym("EISL-IGNORE-TOPLEVEL-CHECK"),Fcons(NIL,NIL)),0);
