#include "fast.h"
static int f_GEN_BROWSE(int arglist, int th);static int GEN_BROWSE(int th);
static int f_BROWSE_INIT(int arglist, int th);static int BROWSE_INIT(int NBS,int M,int NPATS,int IPATS, int th);
