void init_tfunctions(void){
(deftfunc)("PCOEFADD" , f_PCOEFADD);
(deftfunc)("PCPLUS" , f_PCPLUS);
(deftfunc)("PCPLUS1" , f_PCPLUS1);
(deftfunc)("PCTIMES" , f_PCTIMES);
(deftfunc)("PCTIMES1" , f_PCTIMES1);
(deftfunc)("PPLUS" , f_PPLUS);
(deftfunc)("PPLUS1" , f_PPLUS1);
(deftfunc)("PSIMP" , f_PSIMP);
(deftfunc)("PTIMES" , f_PTIMES);
(deftfunc)("PTIMES1" , f_PTIMES1);
(deftfunc)("PTIMES2" , f_PTIMES2);
(deftfunc)("PTIMES3" , f_PTIMES3);
